package wesen.simulator;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.io.*;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.lang.reflect.Method;
import java.lang.ClassLoader;
import com.symantec.itools.javax.swing.JToolBarSeparator;
import com.symantec.itools.javax.swing.icons.ImageIcon;
import com.symantec.itools.javax.swing.borders.BevelBorder;
import com.symantec.itools.javax.swing.borders.CompoundBorder;
import com.symantec.itools.javax.swing.borders.LineBorder;
import com.symantec.itools.javax.swing.borders.SoftBevelBorder;
import com.symantec.itools.javax.swing.borders.EmptyBorder;
import com.symantec.itools.javax.swing.borders.TitledBorder;
import com.symantec.itools.javax.swing.borders.MatteBorder;
import com.symantec.itools.javax.swing.borders.EtchedBorder;
import com.symantec.itools.javax.swing.JMaskedTextField;
import wesen.util.*;
import wesen.robot.ObjectVector;

/**
 * Das GUI der Simulation
 * 
 * @author Wesen-Team
 * @version 1.0
 * 
 * Achtung! Diese Klasse wurde urspruenglich mit Form-Designer von VisualCafe erstellt.
 * Als das GUI mehr oder weniger den Anforderung entsprach wurde der Code ein
 * bisschen "verschoenert".
 * Fuer das Aktivieren bzw. Deaktivieren der Schaltflaechen und Menus sollte vielleicht
 * eine Statemachine eingesetzt werden.
 */
public class Simulator extends JFrame implements Observer {

  /**
   * Ueberschreiben der serialisierungs-ID, damit werden InvalidClassException ausgeschlossen.
   * Besser waere alle nicht fuer die Serialsierung benutzten Variablen als transient zu
   * deklarieren, dann wuerde bei einer Aenderung der nicht relevanten Variablen dieses
   * Problem nicht auftreten.
   */
  static final long                                     serialVersionUID = 
    5229886082550730615L;

  /**
   * Die Kalsse welche das Interface AnimationVector implementiert
   */
  final static String                                   ROBOT_CLASS = 
    "wesen.robot.ObjectVector";

  /**
   * In diese Datei wird die Welt beim Starten bzw. Beenden, auf Benutzerwunsch
   * gespeichert
   */
  final static String                                   STANDARD_WORLDFILE = 
    "wesen.ser";

  /**
   * In diesem Vektor sind alle Aktoren und Roboter gespeichert
   * @supplierCardinality 1
   */
  private AnimationVector                               av;

  /**
   * Referenz auf die Spielflche unserer Simulation
   */

  /**
   * Referenz auf die Klasse welche die Simulation durchfhrt
   * @supplierCardinality 1
   */
  private Simulate                                      simulate;

  /**
   * Ist die Debugfunktion ein- bzw. ausgeschaltet.
   */
  public static boolean                                 withDebug;

  /**
   * Verzeichnis wo diese Applikation gestartet wurde
   */
  private String                                        userDir;

  /**
   * Homeverzeichnis des Benutzers
   */
  private String                                        homeDir;

  /**
   * Datei wo die aktuelle Welt gespeichert wird.
   */
  private String                                        curFile;

  /**
   * Das Zahlenformat des Winkels
   */
  private static DecimalFormat                          dfAngle = 
    new DecimalFormat("000.00");

  /**
   * Speichert den Status des Paste-Buttons beim deaktivieren und dem folgenden wiederherstellen
   */
  private boolean                                       pbPasteState = false;

  /**
   * @supplierCardinality 0..1
   */

  /*# private static Console console; */

  /**
   * Der bisherige Winkel des Objektes, wird gebraucht falls die Eingabe
   * des Benutzer nicht akzeptiert wird.
   */
  private double                                        oldAlpha;

  /**
   * @supplierCardinality 1
   */
  static private wesen.simulator.Umgebung               _umgebung;

  // Hier kommen nur die definitionen der Widgets
  javax.swing.JToolBar                                  JToolBar1 = 
    new javax.swing.JToolBar();
  javax.swing.JButton                                   pbNew = 
    new javax.swing.JButton(new ImageIcon(getClass().getResource("/images/New24.gif")));
  javax.swing.JButton                                   pbOpen = 
    new javax.swing.JButton();
  javax.swing.JButton                                   pbSave = 
    new javax.swing.JButton();
  com.symantec.itools.javax.swing.JToolBarSeparator     JToolBarSeparator1 = 
    new com.symantec.itools.javax.swing.JToolBarSeparator();
  javax.swing.JButton                                   pbCut = 
    new javax.swing.JButton();
  com.symantec.itools.javax.swing.JToolBarSeparator     JToolBarSeparator4 = 
    new com.symantec.itools.javax.swing.JToolBarSeparator();
  javax.swing.JButton                                   pbCopy = 
    new javax.swing.JButton();
  javax.swing.JButton                                   pbPaste = 
    new javax.swing.JButton();
  com.symantec.itools.javax.swing.JToolBarSeparator     JToolBarSeparator2 = 
    new com.symantec.itools.javax.swing.JToolBarSeparator();
  javax.swing.JButton                                   pbReset = 
    new javax.swing.JButton();
  javax.swing.JButton                                   pbStep = 
    new javax.swing.JButton();
  javax.swing.JButton                                   pbPlay = 
    new javax.swing.JButton();
  javax.swing.JButton                                   pbStop = 
    new javax.swing.JButton();
  javax.swing.JToggleButton                             tbTrails = 
    new javax.swing.JToggleButton();
  com.symantec.itools.javax.swing.JToolBarSeparator     JToolBarSeparator3 = 
    new com.symantec.itools.javax.swing.JToolBarSeparator();
  javax.swing.JLabel                                    lbAngle = 
    new javax.swing.JLabel();
  com.symantec.itools.javax.swing.JMaskedTextField      mtfAngle = 
    new com.symantec.itools.javax.swing.JMaskedTextField();
  com.symantec.itools.javax.swing.icons.ImageIcon       newIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       openIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       saveIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       cutIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       copyIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       pasteIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  javax.swing.JPanel                                    pMessage = 
    new javax.swing.JPanel();
  javax.swing.JTextField                                tfMessage1 = 
    new javax.swing.JTextField();
  javax.swing.JMenuBar                                  JMenuBar1 = 
    new javax.swing.JMenuBar();
  javax.swing.JMenu                                     fileMenu = 
    new javax.swing.JMenu();
  javax.swing.JMenuItem                                 newItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenuItem                                 openItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenuItem                                 saveItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenuItem                                 saveAsItem = 
    new javax.swing.JMenuItem();
  javax.swing.JSeparator                                JSeparator1 = 
    new javax.swing.JSeparator();
  javax.swing.JMenuItem                                 exitItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenu                                     editMenu = 
    new javax.swing.JMenu();
  javax.swing.JMenuItem                                 cutItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenuItem                                 copyItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenuItem                                 pasteItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenu                                     actionMenu = 
    new javax.swing.JMenu();
  javax.swing.JMenuItem                                 resetItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenuItem                                 stepItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenuItem                                 playItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenuItem                                 stopItem = 
    new javax.swing.JMenuItem();
  javax.swing.JMenu                                     helpMenu = 
    new javax.swing.JMenu();
  javax.swing.JMenuItem                                 aboutItem = 
    new javax.swing.JMenuItem();
  com.symantec.itools.javax.swing.icons.ImageIcon       resetIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       stepIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       playIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       stopIcon = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       trailIconOn = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.icons.ImageIcon       trailIconOff = 
    new com.symantec.itools.javax.swing.icons.ImageIcon();
  com.symantec.itools.javax.swing.borders.TitledBorder  titledBorder3 = 
    new com.symantec.itools.javax.swing.borders.TitledBorder();
  com.symantec.itools.javax.swing.borders.EtchedBorder  etchedBorder1 = 
    new com.symantec.itools.javax.swing.borders.EtchedBorder();

  /**
   * @supplierCardinality 0..1
   */

  /*# private JAboutDialog lnkJAboutDialog; */

  /**
   * Konstruktor fuer den Simulator-GUI
   */
  public Simulator() {
    try {
      setJMenuBar(JMenuBar1);
      setTitle("Knstliche Wesen");
      setDefaultCloseOperation(javax.swing.JFrame.DO_NOTHING_ON_CLOSE);
      getContentPane().setLayout(new BorderLayout(0, 0));
      setSize(731, 394);
      setVisible(false);
      JToolBar1.setAlignmentY(0.222222F);
      getContentPane().add(BorderLayout.NORTH, JToolBar1);
      pbNew.setDefaultCapable(false);
      pbNew.setToolTipText("Neue Welt erstellen");
      JToolBar1.add(pbNew);
      pbOpen.setDefaultCapable(false);
      pbOpen.setToolTipText("Bestehende Welt laden");
      JToolBar1.add(pbOpen);
      pbSave.setDefaultCapable(false);
      pbSave.setToolTipText("Welt abspeichern");
      JToolBar1.add(pbSave);
      JToolBar1.add(JToolBarSeparator1);
      pbCut.setDefaultCapable(false);
      pbCut.setToolTipText("Lschen des markierten Objektes");
      JToolBar1.add(pbCut);
      JToolBar1.add(JToolBarSeparator4);
      pbCopy.setDefaultCapable(false);
      pbCopy.setToolTipText("Kopieren eines Roboters oder Aktors in die Ablage");
      pbCopy.setEnabled(false);
      JToolBar1.add(pbCopy);
      pbPaste.setDefaultCapable(false);
      pbPaste.setToolTipText("Einfgen aus der Ablage");
      pbPaste.setEnabled(pbPasteState);
      JToolBar1.add(pbPaste);
      JToolBar1.add(JToolBarSeparator2);
      pbReset.setDefaultCapable(false);
      pbReset.setToolTipText("Roboter in Ausgangsposition bringen");
      JToolBar1.add(pbReset);
      pbStep.setDefaultCapable(false);
      pbStep.setToolTipText("Simulation im Schrittmodus durchfhren");
      JToolBar1.add(pbStep);
      pbPlay.setDefaultCapable(false);
      pbPlay.setToolTipText("Starten der Simulation");
      JToolBar1.add(pbPlay);
      pbStop.setDefaultCapable(false);
      pbStop.setToolTipText("Stoppen der Simulation");
      JToolBar1.add(pbStop);
      tbTrails.setContentAreaFilled(false);
      tbTrails.setOpaque(false);
      tbTrails.setToolTipText("Ein- und ausschalten der Bewegungsspur");
      JToolBar1.add(tbTrails);
      JToolBar1.add(JToolBarSeparator3);
      lbAngle.setText("Winkel :");
      lbAngle.setAlignmentX(0.5F);
      JToolBar1.add(lbAngle);
      mtfAngle.setAlignmentX(1.0F);
      mtfAngle.setToolTipText("Bewegungswinkel des markierten Roboters");
      mtfAngle.setMask("990.00");
      mtfAngle.setEditable(false);
      JToolBar1.add(mtfAngle);
      openIcon.setImageLocation(getClass().getResource("/images/Open24.gif"));
      saveIcon.setImageLocation(getClass().getResource("/images/Save24.gif"));
      cutIcon.setImageLocation(getClass().getResource("/images/Delete24.gif"));
      copyIcon.setImageLocation(getClass().getResource("/images/Copy24.gif"));
      pasteIcon.setImageLocation(getClass().getResource("/images/Paste24.gif"));
      resetIcon.setImageLocation(getClass().getResource("/images/Rewind24.gif"));
      stepIcon.setImageLocation(getClass().getResource("/images/StepForward24.gif"));
      playIcon.setImageLocation(getClass().getResource("/images/Play24.gif"));
      stopIcon.setImageLocation(getClass().getResource("/images/Stop24.gif"));
      trailIconOn.setImageLocation(getClass().getResource("/images/TrailsOn24.gif"));
      trailIconOff.setImageLocation(getClass().getResource("/images/TrailsOff24.gif"));

      _umgebung = new wesen.simulator.Umgebung(this);

      _umgebung.setDoubleBuffered(false);
      _umgebung.setLayout(null);
      getContentPane().add(BorderLayout.CENTER, _umgebung);
      _umgebung.setBackground(java.awt.Color.white);
      pMessage.setLayout(new GridBagLayout());
      getContentPane().add(BorderLayout.SOUTH, pMessage);
      pMessage.setBackground(new java.awt.Color(204, 204, 204));
      tfMessage1.setBorder(etchedBorder1);
      tfMessage1.setEditable(false);
      pMessage.add(tfMessage1, 
                   new com.symantec.itools.awt.GridBagConstraintsD(0, 0, 1, 1, 
                   1.0, 1.0, java.awt.GridBagConstraints.CENTER, 
                   java.awt.GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 
                   0));
      fileMenu.setText("Datei");
      fileMenu.setActionCommand("File");
      fileMenu.setBorderPainted(false);
      fileMenu.setMnemonic((int) 'D');
      JMenuBar1.add(fileMenu);
      newItem.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
      newItem.setText("Neu");
      newItem.setActionCommand("New");
      newItem.setBorderPainted(false);
      newItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, 
              java.awt.Event.CTRL_MASK));
      newItem.setMnemonic((int) 'N');
      fileMenu.add(newItem);
      openItem.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
      openItem.setText("ffnen...");
      openItem.setActionCommand("Open...");
      openItem.setBorderPainted(false);
      openItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, 
              java.awt.Event.CTRL_MASK));
      openItem.setMnemonic((int) 'f');
      fileMenu.add(openItem);
      saveItem.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
      saveItem.setText("Speichern");
      saveItem.setActionCommand("Save");
      saveItem.setBorderPainted(false);
      saveItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, 
              java.awt.Event.CTRL_MASK));
      saveItem.setMnemonic((int) 'S');
      fileMenu.add(saveItem);
      saveAsItem.setText("Speichern unter...");
      saveAsItem.setActionCommand("Save As...");
      saveAsItem.setBorderPainted(false);
      saveAsItem.setMnemonic((int) 'u');
      fileMenu.add(saveAsItem);
      fileMenu.add(JSeparator1);
      exitItem.setText("Beenden");
      exitItem.setActionCommand("Exit");
      exitItem.setBorderPainted(false);
      exitItem.setMnemonic((int) 'B');
      fileMenu.add(exitItem);
      editMenu.setText("Bearbeiten");
      editMenu.setActionCommand("Edit");
      editMenu.setBorderPainted(false);
      editMenu.setMnemonic((int) 'B');
      JMenuBar1.add(editMenu);
      cutItem.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
      cutItem.setText("Lschen");
      cutItem.setActionCommand("Cut");
      cutItem.setBorderPainted(false);
      cutItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, 
              java.awt.Event.CTRL_MASK));
      cutItem.setMnemonic((int) 'L');
      editMenu.add(cutItem);
      copyItem.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
      copyItem.setText("Kopieren");
      copyItem.setActionCommand("Copy");
      copyItem.setEnabled(false);
      copyItem.setBorderPainted(false);
      copyItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, 
              java.awt.Event.CTRL_MASK));
      copyItem.setMnemonic((int) 'K');
      editMenu.add(copyItem);
      pasteItem.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
      pasteItem.setText("Einfgen");
      pasteItem.setActionCommand("Paste");
      pasteItem.setBorderPainted(false);
      pasteItem.setEnabled(false);
      pasteItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, 
              java.awt.Event.CTRL_MASK));
      pasteItem.setMnemonic((int) 'P');
      editMenu.add(pasteItem);
      actionMenu.setText("Aktion");
      actionMenu.setActionCommand("Aktion");
      actionMenu.setBorderPainted(false);
      actionMenu.setMnemonic((int) 'A');
      JMenuBar1.add(actionMenu);
      resetItem.setText("Zurckstellen");
      resetItem.setActionCommand("Zurckstellen");
      resetItem.setBorderPainted(false);
      resetItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_B, 
              java.awt.Event.CTRL_MASK));
      resetItem.setMnemonic((int) 'Z');
      actionMenu.add(resetItem);
      stepItem.setText("Einzelschritt");
      stepItem.setActionCommand("Einzelschritt");
      stepItem.setBorderPainted(false);
      stepItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_1, 
              java.awt.Event.CTRL_MASK));
      stepItem.setMnemonic((int) 'E');
      actionMenu.add(stepItem);
      playItem.setText("Start");
      playItem.setActionCommand("Start");
      playItem.setBorderPainted(false);
      playItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, 
              java.awt.Event.CTRL_MASK));
      playItem.setMnemonic((int) 'S');
      actionMenu.add(playItem);
      stopItem.setText("Stop");
      stopItem.setActionCommand("Stop");
      stopItem.setBorderPainted(false);
      stopItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_H, 
              java.awt.Event.CTRL_MASK));
      stopItem.setMnemonic((int) 'P');
      actionMenu.add(stopItem);
      helpMenu.setText("?");
      helpMenu.setActionCommand("Help");
      helpMenu.setBorderPainted(false);
      JMenuBar1.add(helpMenu);
      aboutItem.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
      aboutItem.setText("Info");
      aboutItem.setActionCommand("About...");
      aboutItem.setBorderPainted(false);
      aboutItem.setMnemonic((int) 'o');
      helpMenu.add(aboutItem);
      pbStop.setIcon(stopIcon);
      pbPlay.setIcon(playIcon);
      pbPaste.setIcon(pasteIcon);
      pbCopy.setIcon(copyIcon);
      pbOpen.setIcon(openIcon);
      pbCut.setIcon(cutIcon);
      pbSave.setIcon(saveIcon);

      // pbNew.setIcon(new ImageIcon(getClass().getResource("/images/New24.gif")));
      pbStep.setIcon(stepIcon);
      tbTrails.setSelectedIcon(trailIconOn);
      tbTrails.setIcon(trailIconOff);
      pbReset.setIcon(resetIcon);

      // Listener setzen
      SymWindow aSymWindow = new SymWindow();

      this.addWindowListener(aSymWindow);

      SymAction lSymAction = new SymAction();

      openItem.addActionListener(lSymAction);
      saveItem.addActionListener(lSymAction);
      exitItem.addActionListener(lSymAction);
      aboutItem.addActionListener(lSymAction);
      pbOpen.addActionListener(lSymAction);
      pbSave.addActionListener(lSymAction);
      pbReset.addActionListener(lSymAction);
      pbStep.addActionListener(lSymAction);
      pbPlay.addActionListener(lSymAction);
      pbStop.addActionListener(lSymAction);
      pbNew.addActionListener(lSymAction);
      tbTrails.addActionListener(lSymAction);
      pbCut.addActionListener(lSymAction);
      pbCopy.addActionListener(lSymAction);
      pbPaste.addActionListener(lSymAction);
      mtfAngle.addActionListener(lSymAction);
      cutItem.addActionListener(lSymAction);
      copyItem.addActionListener(lSymAction);
      pasteItem.addActionListener(lSymAction);
      saveAsItem.addActionListener(lSymAction);
      resetItem.addActionListener(lSymAction);
      stepItem.addActionListener(lSymAction);
      playItem.addActionListener(lSymAction);
      stopItem.addActionListener(lSymAction);
      newItem.addActionListener(lSymAction);
      MessageHandler.setResourceFile("properties/error_msg", Locale.GERMAN, 
                                     tfMessage1);

      Class rav = Class.forName(ROBOT_CLASS);

      av = (AnimationVector) rav.newInstance();

      av.registerObserver(this);
      _umgebung.setAnimationVector(av);
      _umgebung.init();
      tbTrails.setPreferredSize(new Dimension(32, 32));
      tbTrails.setMaximumSize(new Dimension(32, 32));
      lbAngle.setPreferredSize(new Dimension(00, 20));
      mtfAngle.setMaximumSize(new Dimension(50, 20));
      mtfAngle.setSpaceIgnored(false);

      simulate = Simulate.getInstance(this, _umgebung);
      userDir = System.getProperty("user.dir");
      homeDir = System.getProperty("user.home");

      /*
       * System.out.println("User Dir: " + userDir);
       * System.out.println("Home Dir: " + homeDir);
       */
      isSomethingToSimulate();

      File  f = new File(homeDir + "/" + STANDARD_WORLDFILE);

      if (f.exists() 
              && JOptionPane.YES_OPTION 
                 == MessageHandler.askQuestion(this, "start.question", 
                                               "confirm", 
                                               JOptionPane.YES_NO_OPTION, 
                                               JOptionPane.QUESTION_MESSAGE)) {
        loadWorld(homeDir, STANDARD_WORLDFILE);
      }   // if
     } catch (Exception e) {
      e.printStackTrace();
    } 
  }

  /**
   * Erstellt eine neue Instanz des Simulator mit sTitle als Titel
   * @param sTitle Titel fuer das Frame
   */
  public Simulator(String sTitle) {
    this();

    setTitle(sTitle);
  }

  /**
   * Beenden der Applikation
   */
  void exitApplication() {
    try {

      Toolkit.getDefaultToolkit().beep();

 
      int reply = MessageHandler.askQuestion(this, "leave.question", "confirm", 
                                             JOptionPane.YES_NO_CANCEL_OPTION, 
                                             JOptionPane.QUESTION_MESSAGE);


      if (reply == JOptionPane.YES_OPTION || reply == JOptionPane.NO_OPTION) {
        if (reply == JOptionPane.YES_OPTION) {

        } 

        this.setVisible(false);
        this.dispose();
        System.exit(0);
      }   // try
     } catch (Exception e) {
      e.printStackTrace();
    } 
  } 

  /**
   * Oeffnet den das Infofenster der Applikation.
   * 
   * @param event Event
   */
  void aboutDialog(java.awt.event.ActionEvent event) {
    try {
      JAboutDialog  JAboutDialog1 = new JAboutDialog(this);

      JAboutDialog1.setModal(true);
      JAboutDialog1.show();
    } catch (Exception e) {
      e.printStackTrace();
    } 
  } 

  /**
   * Die Objekte der Simulationswelt werden in ihre Ursprunkgsposition
   * gesetzt.
   */
  void reset() {
    simulate.simStop();
    av.resetPosition();
    _umgebung.setFlag(1);
    _umgebung.repaint();
  } 

  /**
   * Rueckgabe des Animationsvektores
   * 
   * @return AnimationVektor
   */
  AnimationVector getRoboterVector() {
    return av;
  } 

  /**
   * Rueckgabe der Referenz auf die Simulationswelt
   * 
   * @return Referenz auf die Simualtionswelt
   */
  public static Umgebung getUmgebung() {
    return _umgebung;
  } 

  /**
   * Die Simulationswelt wird geleert, zuerst erfolgt eine Rueckfrage.
   */
  void newAction() {
    int reply = MessageHandler.askQuestion(this, "new.world", "confirm", 
                                           JOptionPane.YES_NO_OPTION, 
                                           JOptionPane.QUESTION_MESSAGE);

    if (reply == JOptionPane.YES_OPTION) {
      av.removeAllElements(this);
      _umgebung.forceRepaint();
    } 
  } 

  /**
   * Oeffnet den Filedialog zum Auswaehlen einer Datei
   */
  private void openFileDialog() {
    try {
      FileDialog  fd = new FileDialog(this);

      fd.setMode(FileDialog.LOAD);
      fd.setFile("*.ser");
      fd.setVisible(true);

      String  theFile = fd.getFile();
      String  theFileDir = fd.getDirectory();

      if (theFile == null || theFile.equals("")) {
        return;
      } 

      File  f = new File(theFileDir, theFile);

      if (!f.exists()) {

        // Datei nicht vorhanden
        return;
      } 

      userDir = theFileDir;
      curFile = theFile;

      loadWorld(userDir, curFile);
    } catch (Exception e) {
      e.printStackTrace();
    } 
  }   // openFileDialog
 
  /**
   * Oeffent den Dateidialog in welchen der Benutzer die Zeildatei auswaehlen kann.
   * 
   * @param forceDialog Bei true wird der Dialog geoeffnet, bei false wird
   * der bestehende Dateiname verwendet.
   */
  private void saveDialog(boolean forceDialog) {
    if (curFile == null || forceDialog) {
      FileDialog  fd = new FileDialog(this);

      if (curFile == null) {
        fd.setFile("*.ser");
      } 

      fd.setDirectory(userDir);
      fd.setMode(FileDialog.SAVE);
      fd.setVisible(true);

      String  theFile = fd.getFile();
      String  theFileDir = fd.getDirectory();

      if ((theFile == null) || (theFile.equals(""))) {
        return;
      } 

      curFile = theFile;
      userDir = theFileDir;
    } 

    saveWorld(userDir, curFile);
  }   // saveDialog
 
  /**
   * Welt in eine Datei speichern
   * 
   * @param dir Dateipfad
   * @param fileName Dateiname
   */
  private void saveWorld(String dir, String fileName) {
    try {
      FileOutputStream    file = new FileOutputStream(dir + "/" + fileName);
      ObjectOutputStream  o = new ObjectOutputStream(file);

      o.writeObject(getBounds());
      o.writeBoolean(tbTrails.isSelected());
      av.saveWorld(o);
      o.flush();
      file.close();
    } catch (IOException ie) {
      ie.printStackTrace();
    }   // catch
   } 

  /**
   * Laden der Welt von einer Datei
   * 
   * @param dir Dateipfad
   * @param fileName Dateiname
   */
  private void loadWorld(String dir, String fileName) {
    try {
      av.removeAllElements(this);

      FileInputStream   file = new FileInputStream(dir + "/" + fileName);
      ObjectInputStream ois = new ObjectInputStream(file);
      Rectangle         r = (Rectangle) ois.readObject();

      this.setBounds(r);
      tbTrails.setSelected(ois.readBoolean());
      av.readWorld(ois);
      file.close();
      show();
      _umgebung.forceRepaint();
    } catch (IOException ie) {
      ie.printStackTrace();
    } catch (ClassNotFoundException cnfe) {
      cnfe.printStackTrace();
    } 
  }   // readWorld
 
  /**
   * Loeschen eines Objektes in der Simulationsumgebung
   */
  private void cut() {
    int markId = av.getMark();

    if (markId != -1) {
      av.removeOneElement(this, markId);
      _umgebung.forceRepaint();
    } 
  }   // cut
 
  /**
   * Objekt der Simulationsumgebung ins Clipboard setzen
   */
  void setClipboard() {
    boolean hasSet = av.setClipboard();

    pbPaste.setEnabled(hasSet);
    pasteItem.setEnabled(hasSet);
  } 

  /**
   * Objekt aus dem Clipboard in die Simulationsumgebung einfuegen.
   */
  void paste() {
    if (av.getClipboard(this, _umgebung)) {

      // Objekt wurde eingefuegt
      _umgebung.repaint();
    }   // if
   }    // paste
 
  /**
   * Der Benutzer drueckte Enter im Eingabefeld fr den Winkel.
   * Das Objekt wird entsprechend angepasst.
   * 
   * @param event Event
   */
  void inputAngle(java.awt.event.ActionEvent event) {
    int markId = av.getMark();

    try {
      double  alpha = dfAngle.parse(mtfAngle.getText()).doubleValue();

      // Setzt den formatierten Winkel in das Feld ein
      mtfAngle.setText(dfAngle.format(alpha));
      av.setAlphaGrad(alpha, markId);
      _umgebung.repaint();
    } catch (ParseException pe) {

      // Kann die Eingabe nicht geparst werden, so setzen wir die bisherige wieder ein.
      mtfAngle.setText(dfAngle.format(Math.toDegrees(oldAlpha) % 360));
    } 
  } 

  /**
   * Aktivert oder deaktiviert das Eingabefeld fuer den Winkel und rechnet
   * den Winkel von Radian in Grad um.
   * 
   * @param isEditable Bei true das Winkelfeld kann ediert werden, bei false
   * der Winkel kann nicht editiert werden.
   * @param alpha Winkel in Radian
   */
  public void setAlpha(boolean isEditable, double alpha) {
    oldAlpha = alpha;

    double  gradAlpha;

    mtfAngle.setEditable(isEditable);

    if ((gradAlpha = Math.toDegrees(alpha) % 360) < 0) {
      gradAlpha = 360 + gradAlpha;
    } 

    mtfAngle.setText(dfAngle.format(gradAlpha));
  } 

  /**
   * Cut- und Copyschaltflaeche bzw. Menuitems werden aktiviert bzw. deaktiviert.
   * 
   * @param isOn Bei true werden diese aktivert, bei false deaktivert.
   */
  public void setCopy(boolean isOn) {
    pbCut.setEnabled(isOn);
    cutItem.setEnabled(isOn);
    pbCopy.setEnabled(isOn);
    copyItem.setEnabled(isOn);
  } 

  /**
   * Dieser Observer wird informiert falls sich die Anzahl der Roboter in der Umgebung
   * aendert. Falls sich keine Roboter in der Umgebung befindet werden die Schaltfleachen
   * fuer die Simulation deaktiviert.
   * 
   * @param obs Das Objekt das beobachtet wird, hier immer NewObservable
   * @param obj Das Objekt das wirklich "beobachtet" wird.
   * 
   * @see java.util.Observer#update(Observable, Object)
   */
  public void update(Observable obs, Object obj) {
    if (obj == av) {
      isSomethingToSimulate();
    }   // if
   } 

  /**
   * Die Simulationsknoepfe aktiviert bzw. deaktiviert entsprechend der
   * Objekte die animiert werden.
   */
  private void isSomethingToSimulate() {

    // Aus- bzw. Einschalten der Simulationsfunktionen
    boolean hasRoboter = av.countNumberOfRoboters() > 0;

    pbStep.setEnabled(hasRoboter);
    stepItem.setEnabled(hasRoboter);
    pbPlay.setEnabled(hasRoboter);
    playItem.setEnabled(hasRoboter);
    pbStop.setEnabled(hasRoboter);
    stopItem.setEnabled(hasRoboter);
    pbReset.setEnabled(hasRoboter);
    resetItem.setEnabled(hasRoboter);
  } 

  /**
   * Starten der Simulation, einige Schaltflaechen werden deaktiviert.
   *
   * @param iStep Anzahl der Zyklen. Fuer Einzelschrittmodus sollte es auf 2 sein 
   * und fuer kontinuierlichen Modus 0.
   */
  private void startSimulation(int iStep) {
    if(iStep == 0) {
      editMenu.setEnabled(false);
      pbCopy.setEnabled(false);

      pbPasteState = pbPaste.isEnabled();

      pbPaste.setEnabled(false);
      pbCut.setEnabled(false);
      pbSave.setEnabled(false);
      pbOpen.setEnabled(false);
      pbNew.setEnabled(false);
      helpMenu.setEnabled(false);
      fileMenu.setEnabled(false);
    }
    simulate.simStart(iStep);
  } 

  /**
   * Stoppen des Simulators. Die Schaltflaechen muessen wieder aktiviert werden
   */
  public void stopSimulation() {
    editMenu.setEnabled(true);
    pbPaste.setEnabled(pbPasteState);
    pbSave.setEnabled(true);
    pbOpen.setEnabled(true);
    pbNew.setEnabled(true);
    helpMenu.setEnabled(true);
    fileMenu.setEnabled(true);
    simulate.simStop();
  } 


  /**
   * Starten des Simulator
   * 
   * @param args -console: Eine programmierte Konsole wird geoeffnet
   * -debug: Eventuell vorhandener Debuginformationen wird ausgegeben
   * und es kann der Objektinspektor pro Objekt gestartet werden.
   * @see
   */
  static public void main(String[] args) {
    int     i = 0;
    String  arg;
    char    flag;

    try {
      while (i < args.length && args[i].startsWith("-")) {
        arg = args[i++];

        // Fuer optionale Wort-Argumente
        if (arg.equalsIgnoreCase("-console")) {
          new Console();
        } else if (arg.equalsIgnoreCase("-debug")) {
          Simulator.withDebug = true;
        } else if (arg.equalsIgnoreCase("-h") || arg.equalsIgnoreCase("-?") 
                   || arg.equalsIgnoreCase("--") 
                   || arg.equalsIgnoreCase("-help")) {
          System.out.println("wesen [-console] [-debug]");
          System.out.println("  -console : Eine Swing-Konsole wird angezeigt");
          System.out.println("  -debug   : Mgliche Ausgabe von Debugdaten");
          System.exit(0);
        } 

        /*
         * // Argumente welche eingegeben werden muessen
         * else if (arg.equals("-output")) {
         * if (i < args.length)
         * outputfile = args[i++];
         * else
         * System.err.println("-output requires a filename");
         * if (vflag)
         * System.out.println("output file = " + outputfile);
         * }
         * 
         * // Pruefen einer Serie von Argumenten
         * else {
         * for (j = 1; j < arg.length(); j++) {
         * flag = arg.charAt(j);
         * switch (flag) {
         * case 'x':
         * if (vflag) System.out.println("Option x");
         * break;
         * case 'n':
         * if (vflag) System.out.println("Option n");
         * break;
         * default:
         * System.err.println("ParseCmdLine: illegal option " + flag);
         * break;
         * } // switch
         * } // for
         * } // else
         */
      }   // while
 
      /*
       * if (i == args.length)
       * System.err.println("Usage: ParseCmdLine [-verbose] [-xn] [-output afile] filename");
       * else
       * System.out.println("Success!");
       */
      Simulator simulator;

      simulator = new Simulator();

      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

      simulator.setLocation((d.width - simulator.getWidth()) / 2, 
                            (d.height - simulator.getHeight()) / 2);
      simulator.setVisible(true);
    } catch (Throwable t) {
      t.printStackTrace();

      // Beenden der Applikation im Fehlerfall
      System.exit(1);
    } 
  }       // main
 
  /**
   * Alle notwendigen Ereignisse werden hier entgegen genommen.
   * 
   * @author Hugo Graf
   */
  class SymAction implements java.awt.event.ActionListener {

    /**
     * Aufgrund des Ereignisses werden die entsprechenden Atkionen ausgefuehrt.
     * 
     * @param event Wird nicht weiter verwendet
     */
    public void actionPerformed(java.awt.event.ActionEvent event) {
      Object  object = event.getSource();

      if (object == openItem) {
        openFileDialog();
      } else if (object == saveItem) {
        saveDialog(false);
      } else if (object == exitItem) {
        exitApplication();
      } else if (object == aboutItem) {
        aboutDialog(event);
      } else if (object == pbOpen) {
        openFileDialog();
      } else if (object == pbSave) {
        saveDialog(false);
      } else if (object == pbReset) {
        reset();
      } else if (object == pbStep) {
        startSimulation(2);
      } else if (object == pbPlay) {
        startSimulation(0);
      } else if (object == pbStop) {
        simulate.simStop();
      } else if (object == pbNew) {
        newAction();
      } else if (object == pbCut) {
        cut();
      } else if (object == pbCopy) {
        setClipboard();
      } else if (object == pbPaste) {
        paste();
      } else if (object == mtfAngle) {
        inputAngle(event);
      } else if (object == cutItem) {
        cut();
      } else if (object == copyItem) {
        setClipboard();
      } else if (object == pasteItem) {
        paste();
      } else if (object == saveAsItem) {
        saveDialog(true);
      } else if (object == resetItem) {
        reset();
      } else if (object == stepItem) {
        startSimulation(2);
      } else if (object == playItem) {
        startSimulation(0);
      } else if (object == stopItem) {
        simulate.simStop();
      } else if (object == newItem) {
        newAction();
      } 
    } 

  }   // class SymAction

  /**
   * Windowslistener
   * 
   * @author Hugo Graf
   */
  class SymWindow extends java.awt.event.WindowAdapter {

    /**
     * Wir behandeln nur das Ereigniss "windowClosing" um an den Benutzer
     * eine Frage zu richten.
     * 
     * @param event Event
     */
    public void windowClosing(java.awt.event.WindowEvent event) {
      Object  object = event.getSource();

      if (object == Simulator.this) {
        exitApplication();
      } 
    }   // windowClosing
 
  }     // SymWindow

}

